# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import pytest
# Allow everything in there to access the DB
pytestmark = pytest.mark.django_db

from django.db import IntegrityError
from django.db.models import ProtectedError
from django.conf import settings
from django.core.exceptions import ValidationError as DjangoValidationError
import ipaddress
import logging

from nsot import exc, models

from .fixtures import admin_user, device, site, user, transactional_db


def test_creation(device):
    """Test basic Interface creation."""
    iface = models.Interface.objects.create(
        device=device, name='eth0'
    )

    # Interface inherits Device's site
    assert iface.site == device.site

    # Name can't be blank
    with pytest.raises(exc.ValidationError):
        iface.name = None
        iface.save()


def test_speed(device):
    """Test interface speed."""
    iface = models.Interface.objects.create(device=device, name='eth0')

    # Interface inherits default speed.
    assert iface.speed == settings.INTERFACE_DEFAULT_SPEED

    # String integers are ok
    iface.speed = '10000'
    iface.save()

    # Bad strings are bad
    with pytest.raises(exc.ValidationError):
        iface.speed = 'bogus'
        iface.save()

    # Floats are also bad
    with pytest.raises(exc.ValidationError):
        iface.speed = 10.0
        iface.save()


def test_mac_address(device):
    """Test mac"""
    iface = models.Interface.objects.create(device=device, name='eth0')
    assert iface.mac_address == settings.INTERFACE_DEFAULT_MAC

    # Bad MAC is bad
    with pytest.raises(exc.ValidationError):
        iface.mac_address = 'bogus'
        iface.save()

    # Set mac by string
    iface.mac_address = '00:00:8E:B1:B5:78'
    iface.save()

    # Set mac by integer
    iface.mac_address = 122191241314
    iface.save()

    assert iface.mac_address == '00:1c:73:2a:60:62'


def test_type(device):
    """Test types."""
    iface = models.Interface.objects.create(device=device, name='eth0')
    assert iface.type == 6

    # Make sure validation works.
    with pytest.raises(exc.ValidationError):
        iface.type = 'bogus'
        iface.save()

    # None is invalid
    with pytest.raises(exc.ValidationError):
        iface.type = None
        iface.save()


def test_attributes(device):
    """Test that attributes work as expected."""
    models.Attribute.objects.create(
        site=device.site, resource_name='Interface', name='vlan'
    )

    iface = models.Interface.objects.create(
        name='eth0', device=device, attributes={'vlan': '300'}
    )

    assert iface.get_attributes() == {'vlan': '300'}

    # Verify that we can zero out attributes
    iface.set_attributes({})
    assert iface.get_attributes() == {}

    # And make sure no bogus attributes can be set.
    with pytest.raises(exc.ValidationError):
        iface.set_attributes(None)

    with pytest.raises(exc.ValidationError):
        iface.set_attributes({0: 'value'})

    with pytest.raises(exc.ValidationError):
        iface.set_attributes({'key': 0})

    with pytest.raises(exc.ValidationError):
        iface.set_attributes({'made_up': 'value'})


def test_set_addresses(device):
    """Test addresses/assignment."""
    root_network = models.Network.objects.create(
        cidr='10.0.0.0/8', site=device.site
    )
    parent_network = models.Network.objects.create(
        cidr='10.1.1.0/24', site=device.site
    )
    iface = models.Interface.objects.create(device=device, name='eth0')

    # Test set_addresses
    addresses = ['10.1.1.1/32']
    iface.set_addresses(addresses)

    # Test get_addresses
    assert iface.get_addresses() == addresses

    # Test get_networks
    assert iface.get_networks() == [str(parent_network)]

    # Set multiple addresses w/ one existing.
    multi = ['10.1.1.1/32', '10.1.1.2/32']
    iface.set_addresses(multi)

    # Multi addresses should match
    assert iface.get_addresses() == multi

    # Set bad address
    with pytest.raises(exc.ValidationError):
        iface.set_addresses(['bogus'])

    # Test overwrite, which should remove all addresses.
    iface.set_addresses([], overwrite=True)
    assert iface.get_addresses() == []


def test_set_addresses_on_create(device):
    """Test address/assignment on create"""
    root_network = models.Network.objects.create(
        cidr='10.0.0.0/8', site=device.site
    )
    parent_network = models.Network.objects.create(
        cidr='10.1.1.0/24', site=device.site
    )
    addresses = ['10.1.1.1/32', '10.1.1.2/32']

    iface = models.Interface.objects.create(
        device=device, name='eth0', addresses=addresses
    )

    # Test get_addresses
    assert iface.get_addresses() == addresses

    # Test get_networks
    assert iface.get_networks() == [str(parent_network)]


def test_assign_address(device):
    root_network = models.Network.objects.create(
        cidr='10.0.0.0/8', site=device.site
    )
    parent_network = models.Network.objects.create(
        cidr='10.1.1.0/24', site=device.site
    )

    iface = models.Interface.objects.create(device=device, name='eth0')

    # Test assign_address
    cidr = '10.1.1.1/32'
    assign1 = iface.assign_address(cidr)
    iface.clean_addresses()

    # Confirm assignments
    assert iface.get_addresses() == [cidr]
    assert assign1.address.cidr == cidr
    assert iface.get_assignments() == [assign1.to_dict()]

    # What if we try to assign an existing address? (The same address will do.)
    with pytest.raises(exc.ValidationError):
        iface.assign_address(cidr)

    # What if we try to assign a non /32 or /128?
    with pytest.raises(exc.ValidationError):
        iface.assign_address('10.1.1.0/28')

    # If we delete assignment, Network object should persist.
    # import pdb; pdb.set_trace()
    address = models.Network.objects.get_by_address(cidr)
    assign1.delete()
    assert models.Network.objects.get_by_address(address.cidr) == address

    # If we delete address, asignment should go away.
    assign2 = iface.assign_address(cidr)
    address.delete()
    assert list(iface.assignments.all()) == []


# TODO(jathan): This isn't implemented yet, but the idea is that there will be
# pluggable parenting/inheritance strategies, with the "SNMP index" strategy as
# the default/built-in (e.g. snmp_index, snmp_parent_index).
def _test_parenting(device):
    pass

    # Test setting interface parent's automatically (by attrs??)

    # Disallow setting non-Interface objects as parent.

# test_retrieve_interfaces



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import pytest

# Allow everything in there to access the DB
pytestmark = pytest.mark.django_db

import copy
from django.core.urlresolvers import reverse
from django.conf import settings
import json
import logging
from rest_framework import status


from .fixtures import live_server, client, user, site
from .util import (
    assert_created, assert_error, assert_success, assert_deleted, load_json,
    Client, load, filter_networks, make_mac, TestSite, get_result
)


log = logging.getLogger(__name__)


def test_network_bug_issues_34(client, site):
    """Test set queries for Networks."""
    # URIs
    attr_uri = site.list_uri('attribute')
    net_uri = site.list_uri('network')

    # Pre-load the attributes
    client.post(attr_uri, data=load('attributes.json'))

    # Populate the network objects and retreive them for testing.
    client.post(net_uri, data=load('networks.json'))
    net_resp = client.retrieve(net_uri)
    networks = get_result(net_resp)

    # Filter networks w/ attribute hostname=foo-bar1, excluding IPs
    wanted = ['192.168.0.0/24', '192.168.0.0/25']
    expected = filter_networks(networks, wanted)

    assert_success(
        client.retrieve(
            net_uri, attributes='hostname=foo-bar1', include_ips=False
        ),
        expected
    )

    # Filter networks w/ attribute hostname=foo-bar1, including IPs
    wanted = ['192.168.0.1/32', '192.168.0.0/24', '192.168.0.0/25']
    expected = filter_networks(networks, wanted)

    assert_success(
        client.retrieve(
            net_uri, attributes='hostname=foo-bar1'
        ),
        expected
    )


def test_mac_address_bug_issues_111(client, site):
    """Test that a MAC coming in as an integer is properly formatted."""
    # Make sure that none of them ever match wrong.
    mac_int = 122191241314
    mac_str = '122191241314'
    mac_expected = '00:1c:73:2a:60:62'
    mac_wrong = '12:21:91:24:13:14'

    dev_uri = site.list_uri('device')
    ifc_uri = site.list_uri('interface')

    dev_resp = client.create(dev_uri, hostname='foo-bar1')
    dev = get_result(dev_resp)

    # Create the interface w/ an integer
    ifc_resp = client.create(
        ifc_uri, device=dev['id'], name='eth0', parent_id=None,
        mac_address=mac_int
    )
    ifc = get_result(ifc_resp)
    ifc_obj_uri = site.detail_uri('interface', id=ifc['id'])

    # Test that integer matches expected
    assert make_mac(ifc['mac_address']) == mac_expected

    # Update the interface w/ a string integer
    updated = copy.deepcopy(ifc)
    updated_resp = client.put(ifc_obj_uri, data=json.dumps(updated))
    expected = get_result(updated_resp)

    # Test that string integer matches expectd
    assert make_mac(expected['mac_address']) == mac_expected

    # And for completeness, make sure that a formatted string still comes back
    # the same.
    updated['mac_address'] = mac_expected
    updated_resp = client.put(ifc_obj_uri, data=json.dumps(updated))
    expected = get_result(updated_resp)

    # Test that expected matches expected
    assert make_mac(expected['mac_address']) == mac_expected


def test_options_bug_issues_126(client, site):
    """
    Test that OPTIONS query returns a 200 OK and has content.

    Ref: https://github.com/dropbox/nsot/issues/126
    """
    net_uri = site.list_uri('network')

    opts_resp = client.options(net_uri)

    # Assert 200 OK
    assert opts_resp.status_code == 200

    # Assert payload is a thing.
    expected = [u'actions', u'description', u'name', u'parses', u'renders']
    assert sorted(opts_resp.json()) == expected


def test_duplicate_400_issues_142(client, site):
    """
    Test that creating duplicate objects results in a 400.

    Ref: https://github.com/dropbox/nsot/issues/142
    """
    dev_uri = site.list_uri('device')
    net_uri = site.list_uri('network')

    # Device is created.
    client.create(dev_uri, hostname='foo-bar1')

    # Device duplicate fails.
    assert_error(
        client.create(dev_uri, hostname='foo-bar1'),
        status.HTTP_400_BAD_REQUEST
    )

    # Network is created.
    client.create(net_uri, cidr='10.0.0.0/8')

    # Network duplicate fails.
    assert_error(
        client.create(net_uri, cidr='10.0.0.0/8'),
        status.HTTP_400_BAD_REQUEST
    )


def test_natural_lookup_without_site(client, site):
    """
    Test that when retrieving objects by natural_key, that uniqueness is
    guaranteed. Since all uniqueness constraints are by Site, we could in
    theory have objects across multiple sites w/ the same natural_key value.

    - You've got multiple sites and an object w/ the same natural_key in
      different Sites. For example: Device 'foo-bar1' in Site 1 and Device
      'foo-bar1' in Site 2.
    - You're not using a site-specific end-point (e.g. /api/devices/ vs.
      /api/sites/1/devices/).
    """
    site1 = site  # For comparison against site2

    # Top-level URIs (e.g. /api/:resource_name/)
    site_uri = reverse('site-list')
    net_uri = reverse('network-list')
    dev_uri = reverse('device-list')

    # Create 2nd site
    site2_resp = client.create(site_uri, name='Test Site 2')
    site2 = TestSite(site2_resp.json())

    ###########
    # Devices #
    ###########
    hostname = 'foo-bar1'
    site1_dev_uri = site1.list_uri('device')
    site2_dev_uri = site2.list_uri('device')

    # Create a Device in each site w/ the same hostname
    client.create(site1_dev_uri, hostname=hostname)
    client.create(site2_dev_uri, hostname=hostname)

    # Site-specific: GOOD
    site1_dev_detail_uri = site1.detail_uri('device', id=hostname)
    site2_dev_detail_uri = site2.detail_uri('device', id=hostname)
    dev1_resp = client.get(site1_dev_detail_uri)
    dev2_resp = client.get(site2_dev_detail_uri)
    assert dev1_resp.status_code == 200
    assert dev2_resp.status_code == 200

    # Top-level: BAD
    root_dev_detail_uri = reverse('device-detail', args=(hostname,))
    assert_error(
        client.get(root_dev_detail_uri),
        status.HTTP_400_BAD_REQUEST
    )

    ############
    # Networks #
    ############
    cidr = '10.0.0.0/8'
    site1_net_uri = site1.list_uri('network')
    site2_net_uri = site2.list_uri('network')

    # Create a Network in each site w/ the same cidr
    client.create(site1_net_uri, cidr=cidr)
    client.create(site2_net_uri, cidr=cidr)

    # Site-specific: GOOD
    site1_net_detail_uri = site1.detail_uri('network', id=cidr)
    site2_net_detail_uri = site2.detail_uri('network', id=cidr)
    net1_resp = client.get(site1_net_detail_uri)
    net2_resp = client.get(site2_net_detail_uri)
    assert net1_resp.status_code == 200
    assert net2_resp.status_code == 200

    # Top-level: BAD
    root_net_detail_uri = reverse('network-detail', args=(cidr,))
    assert_error(
        client.get(root_net_detail_uri),
        status.HTTP_400_BAD_REQUEST
    )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework_bulk.routes import BulkRouter
from rest_framework_nested.routers import NestedSimpleRouter


__all__ = ('BulkRouter', 'BulkNestedRouter')


# Map of HTTP verbs to rest_framework_bulk operations.
BULK_OPERATIONS_MAP = {
    'put': 'bulk_update',
    'patch': 'partial_bulk_update',
    'delete': 'bulk_destroy',
}


class BulkNestedRouter(NestedSimpleRouter):
    """
    Bulk-enabled nested router.
    """
    def __init__(self, *args, **kwargs):
        super(BulkNestedRouter, self).__init__(*args, **kwargs)
        self.routes[0].mapping.update(BULK_OPERATIONS_MAP)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('nsot', '0010_auto_20150921_2120'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='resource',
            name='parent',
        ),
        migrations.AddField(
            model_name='interface',
            name='parent',
            field=models.ForeignKey(related_name='children', on_delete=django.db.models.deletion.PROTECT, default=None, blank=True, to='nsot.Interface', null=True),
        ),
        migrations.AddField(
            model_name='network',
            name='parent',
            field=models.ForeignKey(related_name='children', on_delete=django.db.models.deletion.PROTECT, default=None, blank=True, to='nsot.Network', null=True),
        ),
    ]
